@@ -14,12 +14,26 @@ perls.
     # Basic: run the tests in t/perf/benchmarks against two or
     # more perls
 
-    bench.pl [options] perl1[=label1] perl2[=label2] ...
+    bench.pl [options] -- perlA[=labelA] perlB[=labelB] ...
+
+    # run the tests against same perlA 2x, with and without extra
+    # options
+
+    bench.pl [options] -- perlA=fast PerlA=slow -Mstrict -Dpsltoc 
 
     # Run bench.pl's own built-in sanity tests
 
     bench.pl --action=selftest
 
+    # Run bench on blead, which is then modified and timed again
+
+    bench.pl [options] --write=blead.time -- ./perl=blead
+    # hack hack hack
+    bench.pl --read=blead.time -- ./perl=hacked
+
+    # You can also combine --read with --write
+    bench.pl --read=blead.time --write=last.time -- ./perl=hacked
+
 =head1 DESCRIPTION
 
 By default, F<bench.pl> will run code snippets found in
@@ -30,17 +44,21 @@ perl executables and show how much each test has gotten better or worse.
 
 It is modelled on the F<perlbench> tool, but since it measures instruction
 reads etc., rather than timings, it is much more precise and reproducible.
-It is also considerably faster, and is capable or running tests in
+It is also considerably faster, and is capable of running tests in
 parallel (with C<-j>). Rather than  displaying a single relative
 percentage per test/perl combination, it displays values for 13 different
 measurements, such as instruction reads, conditional branch misses etc.
 
 There are options to write the raw data to a file, and to read it back.
 This means that you can view the same run data in different views with
-different selection and sort options.
+different selection and sort options. You can also use this mechanism
+to save the results of timing one perl, and then read it back while timing
+a modification, so that you dont have rerun the same tests on the same
+perl over and over, or have two perls built at the same time.
 
 The optional C<=label> after each perl executable is used in the display
-output.
+output. If you are doing a two step benchmark then you should provide
+a label for at least the "base" perl.
 
 =head1 OPTIONS
 
@@ -86,6 +104,13 @@ for test I<foo> falls outside the range 100..105.
 
 =item *
 
+--compact=<Iperl>
+
+Display the results for a single perl executable in a compact form.
+Which perl to display is specified in the same manner as C<--norm>.
+
+=item *
+
 --debug
 
 Enable verbose debugging output.
@@ -104,7 +129,70 @@ If only one field is selected, the output is in more compact form.
 
 --grindargs=I<foo>
 
-Optional command-line arguments to pass to cachegrind invocations.
+Optional command-line arguments to pass to all cachegrind invocations.
+
+This option is appended to those which bench.pl uses for its own
+purposes; so it can be used to override them (see --debug output
+below), and can also be 'abused' to add redirects into the valgrind
+command invocation.
+
+For example, this writes PERL_MEM_LOG activity to foobar.$$, because
+3>foobar.$$ redirects fd 3, then perl under PERL_MEM_LOG writes to fd 3.
+
+ $ perl Porting/bench.pl --jobs=2 --verbose --debug \
+    --tests=call::sub::amp_empty \
+    \
+    --grindargs='--cachegrind-out-file=junk.$$ 3>foobar.$$' \
+    -- \
+    perl5.24.0	perl5.24.0:+memlog:PERL_MEM_LOG=3mst
+
+for the +memlog tests, this executes as: (shown via --debug, then prettyfied)
+
+  Command: PERL_HASH_SEED=0 PERL_MEM_LOG=3mst
+    valgrind --tool=cachegrind  --branch-sim=yes
+    --cachegrind-out-file=/dev/null --cachegrind-out-file=junk.$$
+    3>foobar.$$ perl5.24.0  - 10 2>&1
+
+The result is that a set of junk.$$ files containing raw cachegrind
+output are written, and foobar.$$ contains the expected memlog output.
+
+Notes:
+
+Theres no obvious utility for those junk.$$ and foobar.$$ files, but
+you can have them anyway.
+
+The 3 in PERL_MEM_LOG=3mst is needed because the output would
+otherwize go to STDERR, and cause parse_cachegrind() to reject the
+test and die.
+
+The --grindargs redirect is needed to capture the memlog output;
+without it, the memlog output is written to fd3, around
+parse_cachegrind and effectively into /dev/null
+
+PERL_MEM_LOG is expensive when used.
+
+call::sub::amp_empty
+&foo function call with no args or body
+
+       perl5.24.0 perl5.24.0+memlog
+       ---------- -----------------
+    Ir      394.0          543477.5
+    Dr      161.0          146814.1
+    Dw       72.0          122304.6
+  COND       58.0           66796.4
+   IND        5.0            5537.7
+
+COND_m        0.0            6743.1
+ IND_m        5.0            1490.2
+
+ Ir_m1        0.0             683.7
+ Dr_m1        0.0              65.9
+ Dw_m1        0.0               8.5
+
+ Ir_mm        0.0              11.6
+ Dr_mm        0.0              10.6
+ Dw_mm        0.0               4.7
+
 
 =item *
 
@@ -133,8 +221,8 @@ It defaults to the leftmost column.
 
 --perlargs=I<foo>
 
-Optional command-line arguments to pass to each perl that is run as part of
-a cachegrind session. For example, C<--perlargs=-Ilib>.
+Optional command-line arguments to pass to each perl-under-test
+(perlA, perlB in synopsis) For example, C<--perlargs=-Ilib>.
 
 =item *
 
@@ -189,7 +277,9 @@ Display progress information.
 --write=I<file>
 
 Save the raw data to the specified file. It can be read back later with
-C<--read>.
+C<--read>. If combined with C<--read> then the output file will be
+the merge of the file read and any additional perls added on the command
+line.
 
 Requires C<JSON::PP> to be available.
 
@@ -202,7 +292,7 @@ Requires C<JSON::PP> to be available.
 use 5.010000;
 use warnings;
 use strict;
-use Getopt::Long qw(:config no_auto_abbrev);
+use Getopt::Long qw(:config no_auto_abbrev require_order);
 use IPC::Open2 ();
 use IO::Select;
 use IO::File;
@@ -220,7 +310,7 @@ my %VALID_FIELDS = map { $_ => 1 }
 
 sub usage {
     die <<EOF;
-usage: $0 [options] perl[=label] ...
+usage: $0 [options] -- perl[=label] ...
   --action=foo       What action to perform [default: grind].
   --average          Only display average, not individual test results.
   --benchfile=foo    File containing the benchmarks;
@@ -228,6 +318,8 @@ usage: $0 [options] perl[=label] ...
   --bisect=f,min,max run a single test against one perl and exit with a
                        zero status if the named field is in the specified
                        range; exit 1 otherwise.
+  --compact=perl     Display the results of a single perl in compact form.
+                     Which perl specified like --norm
   --debug            Enable verbose debugging output.
   --fields=a,b,c     Display only the specified fields (e.g. Ir,Ir_m,Ir_mm).
   --grindargs=foo    Optional command-line args to pass to cachegrind.
@@ -238,6 +330,7 @@ usage: $0 [options] perl[=label] ...
                        [default: 0].
   --perlargs=foo     Optional command-line args to pass to each perl to run.
   --raw              Display raw data counts rather than percentages.
+  --show             Show results even though we are going to write results.
   --sort=field:perl  Sort the tests based on the value of 'field' in the
                        column 'perl'. The perl value is as per --norm.
   -r|--read=file     Read in previously saved data from the specified file.
@@ -265,6 +358,7 @@ my %OPTS = (
     average   => 0,
     benchfile => 't/perf/benchmarks',
     bisect    => undef,
+    compact   => undef,
     debug     => 0,
     grindargs => '',
     fields    => undef,
@@ -273,6 +367,7 @@ my %OPTS = (
     perlargs  => '',
     raw       => 0,
     read      => undef,
+    show      => 0,
     sort      => undef,
     tests     => undef,
     verbose   => 0,
@@ -288,6 +383,7 @@ my %OPTS = (
         'average'     => \$OPTS{average},
         'benchfile=s' => \$OPTS{benchfile},
         'bisect=s'    => \$OPTS{bisect},
+        'compact=s'   => \$OPTS{compact},
         'debug'       => \$OPTS{debug},
         'grindargs=s' => \$OPTS{grindargs},
         'help'        => \$OPTS{help},
@@ -297,6 +393,7 @@ my %OPTS = (
         'perlargs=s'  => \$OPTS{perlargs},
         'raw'         => \$OPTS{raw},
         'read|r=s'    => \$OPTS{read},
+        'show!'       => \$OPTS{show},
         'sort=s'      => \$OPTS{sort},
         'tests=s'     => \$OPTS{tests},
         'verbose'     => \$OPTS{verbose},
@@ -306,10 +403,6 @@ my %OPTS = (
     usage if $OPTS{help};
 
 
-    if (defined $OPTS{read} and defined $OPTS{write}) {
-        die "Error: can't specify both --read and --write options\n";
-    }
-
     if (defined $OPTS{read} or defined $OPTS{write}) {
         # fail early if it's not present
         require JSON::PP;
@@ -359,21 +452,6 @@ my %OPTS = (
         die "Error: Can't specify both --bisect and --write\n"
                                                 if defined $OPTS{write};
     }
-    elsif (defined $OPTS{read}) {
-        if (@ARGV) {
-            die "Error: no perl executables may be specified with --read\n"
-        }
-    }
-    elsif ($OPTS{raw}) {
-        unless (@ARGV) {
-            die "Error: at least one perl executable must be specified\n";
-        }
-    }
-    else {
-        unless (@ARGV >= 2) {
-            die "Error: at least two perl executables must be specified\n";
-        }
-    }
 
     if ($OPTS{action} eq 'grind') {
         do_grind(\@ARGV);
@@ -406,30 +484,44 @@ sub filter_tests {
     else {
         my %t;
         for (split /,/, $opt) {
-            die "Error: no such test found: '$_'\n" unless exists $tests->{$_};
+            die "Error: no such test found: '$_'\n"
+                . ($OPTS{verbose} ? "  have: @{[ sort keys %$tests ]}\n" : "")
+                unless exists $tests->{$_};
             $t{$_} = 1;
         }
         for (keys %$tests) {
             delete $tests->{$_} unless exists $t{$_};
         }
     }
+    die "Error: no tests to run\n" unless %$tests;
 }
 
 
 # Read in the test file, and filter out any tests excluded by $OPTS{tests}
+# return a hash ref { testname => { test }, ... }
+# and an array ref of the original test names order,
 
 sub read_tests_file {
     my ($file) = @_;
 
-    my $ta = do $file;
+    my $ta;
+    {
+        local @INC = ('.');
+        $ta = do $file;
+    }
     unless ($ta) {
         die "Error: can't parse '$file': $@\n" if $@;
         die "Error: can't read '$file': $!\n";
     }
 
+    my @orig_order;
+    for (my $i=0; $i < @$ta; $i += 2) {
+        push @orig_order, $ta->[$i];
+    }
+
     my $t = { @$ta };
     filter_tests($t);
-    return $t;
+    return $t, \@orig_order;
 }
 
 
@@ -438,7 +530,7 @@ sub read_tests_file {
 
 sub select_a_perl {
     my ($perl, $perls, $who) = @_;
-
+    $perls||=[];
     if ($perl =~ /^[0-9]$/) {
         die "Error: $who value $perl outside range 0.." . $#$perls . "\n"
                                         unless $perl < @$perls;
@@ -457,19 +549,41 @@ sub select_a_perl {
 }
 
 
-# Validate the list of perl=label on the command line.
-# Return a list of [ exe, label ] pairs.
+# Validate the list of perl=label (+ cmdline options) on the command line.
+# Return a list of [ exe, label, cmdline-options ] tuples, i.e.
+# 'perl-under-test's (PUTs)
 
-sub process_perls {
-    my @results;
-    for my $p (@_) {
-        my ($perl, $label) = split /=/, $p, 2;
+sub process_puts {
+    my $read_perls= shift;
+    my @res_puts; # returned, each item is [ perlexe, label, @putargs ]
+    my %seen= map { $_->[1] => 1 } @$read_perls;
+    my @putargs; # collect not-perls into args per PUT
+
+    for my $p (reverse @_) {
+        push @putargs, $p and next if $p =~ /^-/; # not-perl, dont send to qx//
+
+        my ($perl, $label, $env) = split /[=:,]/, $p, 3;
         $label //= $perl;
+        $label = $perl.$label if $label =~ /^\+/;
+        die "$label cannot be used on 2 different perls under test\n" if $seen{$label}++;
+
+        my %env;
+        if ($env) {
+            %env = split /[=,]/, $env;
+        }
         my $r = qx($perl -e 'print qq(ok\n)' 2>&1);
-        die "Error: unable to execute '$perl': $r" if $r ne "ok\n";
-        push @results, [ $perl, $label ];
+        if ($r eq "ok\n") {
+	    push @res_puts, [ $perl, $label, \%env, reverse @putargs ];
+            @putargs = ();
+            warn "Added Perl-Under-Test: [ @{[@{$res_puts[-1]}]} ]\n"
+                if $OPTS{verbose};
+	} else {
+            warn "perl-under-test args: @putargs + a not-perl: $p $r\n"
+                if $OPTS{verbose};
+            push @putargs, $p; # not-perl
+	}
     }
-    return @results;
+    return reverse @res_puts;
 }
 
 
@@ -547,7 +661,7 @@ sub parse_cachegrind {
 sub do_grind {
     my ($perl_args) = @_; # the residue of @ARGV after option processing
 
-    my ($loop_counts, $perls, $results, $tests);
+    my ($loop_counts, $perls, $results, $tests, $order);
     my ($bisect_field, $bisect_min, $bisect_max);
 
     if (defined $OPTS{bisect}) {
@@ -565,9 +679,9 @@ sub do_grind {
             if $bisect_min > $bisect_max;
     }
 
-    if (defined $OPTS{read}) {
+    if ($OPTS{read}) {
         open my $in, '<:encoding(UTF-8)', $OPTS{read}
-            or die " Error: can't open $OPTS{read} for reading: $!\n";
+            or die " Error: can't open '$OPTS{read}' for reading: $!\n";
         my $data = do { local $/; <$in> };
         close $in;
 
@@ -576,29 +690,40 @@ sub do_grind {
             die "Error: unsupported version $hash->{version} in file"
               . "'$OPTS{read}' (too new)\n";
         }
-        ($loop_counts, $perls, $results, $tests) =
-            @$hash{qw(loop_counts perls results tests)};
+        ($loop_counts, $perls, $results, $tests, $order) =
+            @$hash{qw(loop_counts perls results tests order)};
 
         filter_tests($results);
         filter_tests($tests);
-    }
-    else {
-        # How many times to execute the loop for the two trials. The lower
-        # value is intended to do the loop enough times that branch
-        # prediction has taken hold; the higher loop allows us to see the
-        # branch misses after that
-        $loop_counts = [10, 20];
 
-        $tests = read_tests_file($OPTS{benchfile});
-        die "Error: only a single test may be specified with --bisect\n"
-            if defined $OPTS{bisect} and keys %$tests != 1;
-
-        $perls = [ process_perls(@$perl_args) ];
-
-
-        $results = grind_run($tests, $perls, $loop_counts);
+        if (!$order) {
+            $order = [ sort keys %$tests ];
+        }
     }
 
+    if (@$perl_args) {
+        unless ($loop_counts) {
+            # How many times to execute the loop for the two trials. The lower
+            # value is intended to do the loop enough times that branch
+            # prediction has taken hold; the higher loop allows us to see the
+            # branch misses after that
+            $loop_counts = [10, 20];
+
+            ($tests, $order) = read_tests_file($OPTS{benchfile});
+            die "Error: only a single test may be specified with --bisect\n"
+                if defined $OPTS{bisect} and keys %$tests != 1;
+        }
+
+        my @run_perls= process_puts($perls, @$perl_args);
+        push @$perls, @run_perls;
+        die "Error: Not enough perls to run a report, and --write not specified.\n"
+            if @$perls < 2 and !$OPTS{write};
+        $results = grind_run($tests, $order, \@run_perls, $loop_counts, $results);
+    }
+
+    if (!$perls or !@$perls) {
+        die "Error: nothing to do: no perls to run, no data to read.\n";
+    }
     # now that we have a list of perls, use it to process the
     # 'perl' component of the --norm and --sort args
 
@@ -608,6 +733,10 @@ sub do_grind {
                 select_a_perl($OPTS{'sort-perl'}, $perls, "--sort");
     }
 
+    if (defined $OPTS{'compact'}) {
+        $OPTS{'compact'} =
+                select_a_perl($OPTS{'compact'}, $perls, "--compact");
+    }
     if (defined $OPTS{write}) {
         my $json = JSON::PP::encode_json({
                     version      => $FORMAT_VERSION,
@@ -615,14 +744,18 @@ sub do_grind {
                     perls        => $perls,
                     results      => $results,
                     tests        => $tests,
+                    order        => $order,
                 });
 
         open my $out, '>:encoding(UTF-8)', $OPTS{write}
-            or die " Error: can't open $OPTS{write} for writing: $!\n";
+            or die "Error: can't open '$OPTS{write}' for writing: $!\n";
         print $out $json or die "Error: writing to file '$OPTS{write}': $!\n";
         close $out       or die "Error: closing file '$OPTS{write}': $!\n";
     }
-    else {
+    if (!$OPTS{write} or $OPTS{show}) {
+        if (@$perls < 2) {
+            die "Error: need more than one perl to do a report.\n";
+        }
         my ($processed, $averages) =
                     grind_process($results, $perls, $loop_counts);
 
@@ -639,8 +772,12 @@ sub do_grind {
             exit 0 if $bisect_min <= $c and $c <= $bisect_max;
             exit 1;
         }
+        elsif (defined $OPTS{compact}) {
+            grind_print_compact($processed, $averages, $OPTS{compact},
+                                $perls, $tests, $order);
+        }
         else {
-            grind_print($processed, $averages, $perls, $tests);
+            grind_print($processed, $averages, $perls, $tests, $order);
         }
     }
 }
@@ -651,13 +788,13 @@ sub do_grind {
 # Return a hash ref suitable for input to grind_process()
 
 sub grind_run {
-    my ($tests, $perls, $counts) = @_;
+    my ($tests, $order, $perls, $counts, $results) = @_;
 
     # Build a list of all the jobs to run
 
     my @jobs;
 
-    for my $test (sort keys %$tests) {
+    for my $test (grep $tests->{$_}, @$order) {
 
         # Create two test progs: one with an empty loop and one with code.
         # Note that the empty loop is actually '{1;}' rather than '{}';
@@ -671,20 +808,24 @@ sub grind_run {
         );
 
         for my $p (@$perls) {
-            my ($perl, $label) = @$p;
+            my ($perl, $label, $env, @putargs) = @$p;
 
             # Run both the empty loop and the active loop
             # $counts->[0] and $counts->[1] times.
 
             for my $i (0,1) {
                 for my $j (0,1) {
-                    my $cmd = "PERL_HASH_SEED=0 "
+                    my $envstr = '';
+                    if (ref $env) {
+                        $envstr .= "$_=$env->{$_} " for sort keys %$env;
+                    }
+                    my $cmd = "PERL_HASH_SEED=0 $envstr"
                             . "valgrind --tool=cachegrind  --branch-sim=yes "
                             . "--cachegrind-out-file=/dev/null "
                             . "$OPTS{grindargs} "
-                            . "$perl $OPTS{perlargs} - $counts->[$j] 2>&1";
+                            . "$perl $OPTS{perlargs} @putargs - $counts->[$j] 2>&1";
                     # for debugging and error messages
-                    my $id = "$test/$perl "
+                    my $id = "$test/$label "
                         . ($i ? "active" : "empty") . "/"
                         . ($j ? "long"   : "short") . " loop";
 
@@ -711,7 +852,6 @@ sub grind_run {
     my $running  = 0; # count of executing jobs
     my %pids;         # map pids to jobs
     my %fds;          # map fds  to jobs
-    my %results;
     my $select = IO::Select->new();
 
     while (@jobs or $running) {
@@ -812,7 +952,7 @@ sub grind_run {
                     . "Output\n$o";
             }
 
-            $results{$j->{test}}{$j->{perl}}[$j->{active}][$j->{loopix}]
+            $results->{$j->{test}}{$j->{plabel}}[$j->{active}][$j->{loopix}]
                     = parse_cachegrind($output, $j->{id}, $j->{perl});
         }
 
@@ -835,7 +975,7 @@ sub grind_run {
         }
     }
 
-    return \%results;
+    return $results;
 }
 
 
@@ -901,7 +1041,7 @@ sub grind_process {
     my %counts;
     my %data;
 
-    my $perl_norm = $perls->[$OPTS{norm}][0]; # the name of the reference perl
+    my $perl_norm = $perls->[$OPTS{norm}][1]; # the label of the reference perl
 
     for my $test_name (keys %$res) {
         my $res1 = $res->{$test_name};
@@ -983,28 +1123,11 @@ sub grind_process {
 }
 
 
-# grind_print(): display the tabulated results of all the cachegrinds.
-#
-# Arguments are of the form:
-#    $results->{benchmark_name}{perl_name}{field_name} = N
-#    $averages->{perl_name}{field_name} = M
-#    $perls = [ [ perl-exe, perl-label ], ... ]
-#    $tests->{test_name}{desc => ..., ...}
 
-sub grind_print {
-    my ($results, $averages, $perls, $tests) = @_;
+# print a standard blurb at the start of the grind display
 
-    my @perl_names = map $_->[0], @$perls;
-    my %perl_labels;
-    $perl_labels{$_->[0]} = $_->[1] for @$perls;
-
-    my $field_label_width = 6;
-    # Calculate the width to display for each column.
-    my $min_width = $OPTS{raw} ? 8 : 6;
-    my @widths = map { length($_) < $min_width ? $min_width : length($_) }
-                            @perl_labels{@perl_names};
-
-    # Print header.
+sub grind_blurb {
+    my ($perls) = @_;
 
     print <<EOF;
 Key:
@@ -1026,20 +1149,25 @@ EOF
     else {
         print <<EOF;
 The numbers represent relative counts per loop iteration, compared to
-$perl_labels{$perl_names[0]} at 100.0%.
+$perls->[$OPTS{norm}][1] at 100.0%.
 Higher is better: for example, using half as many instructions gives 200%,
 while using twice as many gives 50%.
 EOF
     }
+}
 
-    # Populate @test_names with the tests in sorted order.
 
-    my @test_names;
+# return a sorted list of the test names, plus 'AVERAGE'
+
+sub sorted_test_names {
+    my ($results, $order, $perls) = @_;
+
+    my @names;
     unless ($OPTS{average}) {
         if (defined $OPTS{'sort-field'}) {
             my ($field, $perlix) = @OPTS{'sort-field', 'sort-perl'};
-            my $perl = $perls->[$perlix][0];
-            @test_names = sort
+            my $perl = $perls->[$perlix][1];
+            @names = sort
                 {
                         $results->{$a}{$perl}{$field}
                     <=> $results->{$b}{$perl}{$field}
@@ -1047,12 +1175,42 @@ EOF
                 keys %$results;
         }
         else {
-            @test_names = sort(keys %$results);
+            @names = grep $results->{$_}, @$order;
         }
     }
 
     # No point in displaying average for only one test.
-    push @test_names,  'AVERAGE' unless @test_names == 1;
+    push @names,  'AVERAGE' unless @names == 1;
+    @names;
+}
+
+
+# grind_print(): display the tabulated results of all the cachegrinds.
+#
+# Arguments are of the form:
+#    $results->{benchmark_name}{perl_name}{field_name} = N
+#    $averages->{perl_name}{field_name} = M
+#    $perls = [ [ perl-exe, perl-label ], ... ]
+#    $tests->{test_name}{desc => ..., ...}
+
+sub grind_print {
+    my ($results, $averages, $perls, $tests, $order) = @_;
+
+    my @perl_names = map $_->[0], @$perls;
+    my @perl_labels = map $_->[1], @$perls;
+    my %perl_labels;
+    $perl_labels{$_->[0]} = $_->[1] for @$perls;
+
+    my $field_label_width = 6;
+    # Calculate the width to display for each column.
+    my $min_width = $OPTS{raw} ? 8 : 6;
+    my @widths = map { length($_) < $min_width ? $min_width : length($_) }
+    			@perl_labels;
+
+    # Print standard header.
+    grind_blurb($perls);
+
+    my @test_names = sorted_test_names($results, $order, $perls);
 
     # If only a single field is to be displayed, use a more compact
     # format with only a single line of output per test.
@@ -1077,7 +1235,7 @@ EOF
             print " " x $field_label_width;
             for (0..$#widths) {
                 printf " %*s", $widths[$_],
-                    $i ? ('-' x$widths[$_]) :  $perl_labels{$perl_names[$_]};
+                    $i ? ('-' x$widths[$_]) :  $perl_labels[$_];
             }
             print "\n";
         }
@@ -1099,7 +1257,7 @@ EOF
                 print " " x $field_label_width;
                 for (0..$#widths) {
                     printf " %*s", $widths[$_],
-                        $i ? ('-' x$widths[$_]) :  $perl_labels{$perl_names[$_]};
+                        $i ? ('-' x$widths[$_]) :  $perl_labels[$_];
                 }
                 print "\n";
             }
@@ -1129,7 +1287,7 @@ EOF
             }
 
             for my $i (0..$#widths) {
-                my $res2 = $res1->{$perl_names[$i]};
+                my $res2 = $res1->{$perl_labels[$i]};
                 my $p = $res2->{$field};
                 if (!defined $p) {
                     printf " %*s", $widths[$i], '-';
@@ -1147,6 +1305,72 @@ EOF
 }
 
 
+
+# grind_print_compact(): like grind_print(), but display a single perl
+# in a compact form. Has an additional arg, $which_perl, which specifies
+# which perl to display.
+#
+# Arguments are of the form:
+#    $results->{benchmark_name}{perl_name}{field_name} = N
+#    $averages->{perl_name}{field_name} = M
+#    $perls = [ [ perl-exe, perl-label ], ... ]
+#    $tests->{test_name}{desc => ..., ...}
+
+sub grind_print_compact {
+    my ($results, $averages, $which_perl, $perls, $tests, $order) = @_;
+
+
+    # the width to display for each column.
+    my $width = $OPTS{raw} ? 7 : 6;
+
+    # Print standard header.
+    grind_blurb($perls);
+
+    print "\nResults for $perls->[$which_perl][1]\n\n";
+
+    my @test_names = sorted_test_names($results, $order, $perls);
+
+    # Dump the results for each test.
+
+     my @fields = qw( Ir Dr Dw
+                      COND IND
+                      COND_m IND_m
+                      Ir_m1 Dr_m1 Dw_m1
+                      Ir_mm Dr_mm Dw_mm
+                    );
+    if ($OPTS{fields}) {
+        @fields = grep exists $OPTS{fields}{$_}, @fields;
+    }
+
+    printf " %*s", $width, $_      for @fields;
+    print "\n";
+    printf " %*s", $width, '------' for @fields;
+    print "\n";
+
+    for my $test_name (@test_names) {
+        my $doing_ave = ($test_name eq 'AVERAGE');
+        my $res = $doing_ave ? $averages : $results->{$test_name};
+        $res = $res->{$perls->[$which_perl][1]};
+
+        for my $field (@fields) {
+            my $p = $res->{$field};
+            if (!defined $p) {
+                printf " %*s", $width, '-';
+            }
+            elsif ($OPTS{raw}) {
+                printf " %*.1f", $width, $p;
+            }
+            else {
+                printf " %*.2f", $width, $p * 100;
+            }
+
+        }
+
+        print "  $test_name\n";
+    }
+}
+
+
 # do_selftest(): check that we can parse known cachegrind()
 # output formats. If the output of cachegrind changes, add a *new*
 # test here; keep the old tests to make sure we continue to parse
@@ -1201,8 +1425,10 @@ EOF
         },
     );
 
-    for ('t', '.') {
-        last if require "$_/test.pl";
+    for ('./t', '.') {
+        my $t = "$_/test.pl";
+        next unless  -f $t;
+        require $t;
     }
     plan(@tests / 3 * keys %VALID_FIELDS);
 
