@@ -10,7 +10,7 @@ use File::Path ();
 use File::Spec ();
 use CPAN::Mirrors ();
 use vars qw($VERSION $auto_config);
-$VERSION = "5.5304";
+$VERSION = "5.5310";
 
 =head1 NAME
 
@@ -124,6 +124,14 @@ checks will be performed at all.
 Always try to check and verify signatures if a SIGNATURE file is in
 the package and Module::Signature is installed (yes/no)?
 
+=item cleanup_after_install
+
+Users who install modules and do not intend to look back, can free
+occupied disk space quickly by letting CPAN.pm cleanup each build
+directory immediately after a successful install.
+
+Remove build directory after a successful install? (yes/no)?
+
 =item colorize_output
 
 When you have Term::ANSIColor installed, you can turn on colorized
@@ -132,6 +140,9 @@ warnings, debugging output, and the output of the modules being
 installed. Set your favorite colors after some experimenting with the
 Term::ANSIColor module.
 
+Please note that on Windows platforms colorized output also requires
+the Win32::Console::ANSI module.
+
 Do you want to turn on colored output?
 
 =item colorize_print
@@ -195,6 +206,7 @@ alternatives can be configured according to the following table:
     cwd         Cwd::cwd
     getcwd      Cwd::getcwd
     fastcwd     Cwd::fastcwd
+    getdcwd     Cwd::getdcwd
     backtickcwd external command cwd
 
 Preferred method for determining the current working directory?
@@ -203,7 +215,8 @@ Preferred method for determining the current working directory?
 
 Normally, CPAN.pm continues processing the full list of targets and
 dependencies, even if one of them fails.  However, you can specify
-that CPAN should halt after the first failure.
+that CPAN should halt after the first failure.  (Note that optional
+recommended or suggested modules that fail will not cause a halt.)
 
 Do you want to halt on failure (yes/no)?
 
@@ -420,6 +433,14 @@ host should be tried first.
 
 Randomize parameter
 
+=item recommends_policy
+
+(Experimental feature!) Some CPAN modules recommend additional, optional dependencies.  These should
+generally be installed except in resource constrained environments.  When this
+policy is true, recommended modules will be included with required modules.
+
+Included recommended modules?
+
 =item scan_cache
 
 By default, each time the CPAN module is started, cache scanning is
@@ -462,6 +483,14 @@ variable.
 
 Show all individual modules that have a $VERSION of zero?
 
+=item suggests_policy
+
+(Experimental feature!) Some CPAN modules suggest additional, optional dependencies.  These 'suggest'
+dependencies provide enhanced operation.  When this policy is true, suggested
+modules will be included with required modules.
+
+Included suggested modules?
+
 =item tar_verbosity
 
 When CPAN.pm uses the tar command, which switch for the verbosity
@@ -538,6 +567,17 @@ regardless of the history using "force".
 
 Do you want to rely on the test report history (yes/no)?
 
+=item use_prompt_default
+
+When this is true, CPAN will set PERL_MM_USE_DEFAULT to a true
+value.  This causes ExtUtils::MakeMaker (and compatible) prompts
+to use default values instead of stopping to prompt you to answer
+questions. It also sets NONINTERACTIVE_TESTING to a true value to
+signal more generally that distributions should not try to
+interact with you.
+
+Do you want to use prompt defaults (yes/no)?
+
 =item use_sqlite
 
 CPAN::SQLite is a layer between the index files that are downloaded
@@ -743,7 +783,7 @@ sub init {
         }
     } elsif (0 == length $matcher) {
     } elsif (0 && $matcher eq "~") { # extremely buggy, but a nice idea
-        my @unconfigured = grep { not exists $CPAN::Config->{$_}
+        my @unconfigured = sort grep { not exists $CPAN::Config->{$_}
                                       or not defined $CPAN::Config->{$_}
                                           or not length $CPAN::Config->{$_}
                                   } keys %$CPAN::Config;
@@ -849,6 +889,7 @@ sub init {
 
     my_dflt_prompt(index_expire => 1, $matcher);
     my_prompt_loop(scan_cache => 'atstart', $matcher, 'atstart|atexit|never');
+    my_yn_prompt(cleanup_after_install => 0, $matcher);
 
     #
     #= cache_metadata
@@ -865,6 +906,8 @@ sub init {
                    'follow|ask|ignore');
     my_prompt_loop(build_requires_install_policy => 'yes', $matcher,
                    'yes|no|ask/yes|ask/no');
+    my_yn_prompt(recommends_policy => 1, $matcher);
+    my_yn_prompt(suggests_policy => 0, $matcher);
 
     #
     #= Module::Signature
@@ -1045,6 +1088,11 @@ sub init {
 
     my_dflt_prompt(mbuild_install_arg => "", $matcher);
 
+    #
+    #== use_prompt_default
+    #
+    my_yn_prompt(use_prompt_default => 0, $matcher);
+
     #
     #= Alarm period
     #
@@ -1095,6 +1143,17 @@ sub init {
         }
     }
 
+    #
+    #= how plugins work
+    #
+
+    # XXX MISSING: my_array_prompt to be used with plugins. We did something like this near
+    #     git log -p fd68f8f5e33f4cecea4fdb7abc5ee19c12f138f0..test-notest-test-dependency
+    # Need to do similar steps for plugin_list. As long as we do not support it here, people
+    # must use the cpan shell prompt to write something like
+    #     o conf plugin_list push CPAN::Plugin::Specfile=dir,/tmp/foo-20141013,...
+    #     o conf commit
+
     #
     #= how FTP works
     #
@@ -1106,7 +1165,7 @@ sub init {
     #
 
     my_prompt_loop(getcwd => 'cwd', $matcher,
-                   'cwd|getcwd|fastcwd|backtickcwd');
+                   'cwd|getcwd|fastcwd|getdcwd|backtickcwd');
 
     #
     #= the CPAN shell itself (prompt, color)
@@ -1224,10 +1283,7 @@ sub init {
             );
         }
         else {
-            $CPAN::Frontend->myprint(
-                "Autoconfigured everything but 'urllist'.\n"
-            );
-            _do_pick_mirrors();
+            $CPAN::Config->{urllist} = [ 'http://www.cpan.org/' ];
         }
     }
     elsif (!$matcher || "urllist" =~ $matcher) {
@@ -1253,8 +1309,9 @@ sub init {
             $CPAN::Frontend->myprint("\nWriting $configpm for bootstrap...\n");
             delete $CPAN::Config->{install_help}; # temporary only
             CPAN::HandleConfig->commit;
-            my $dist;
-            if ( $dist = CPAN::Shell->expand('Module', 'local::lib')->distribution ) {
+            my($dist, $locallib);
+            $locallib = CPAN::Shell->expand('Module', 'local::lib');
+            if ( $locallib and $dist = $locallib->distribution ) {
                 # this is a hack to force bootstrapping
                 $dist->{prefs}{pl}{commandline} = "$^X Makefile.PL --bootstrap";
                 # Set @INC for this process so we find things as they bootstrap
@@ -1298,10 +1355,6 @@ sub init {
 sub _local_lib_config {
     # Set environment stuff for this process
     require local::lib;
-    my %env = local::lib->build_environment_vars_for(_local_lib_path(), 1);
-    while ( my ($k, $v) = each %env ) {
-        $ENV{$k} = $v;
-    }
 
     # Tell user about environment vars to set
     $CPAN::Frontend->myprint($prompts{local_lib_installed});
@@ -1309,6 +1362,12 @@ sub _local_lib_config {
     my $shellvars = local::lib->environment_vars_string_for(_local_lib_path());
     $CPAN::Frontend->myprint($shellvars);
 
+    # Set %ENV after getting string above
+    my %env = local::lib->build_environment_vars_for(_local_lib_path(), 1);
+    while ( my ($k, $v) = each %env ) {
+        $ENV{$k} = $v;
+    }
+
     # Offer to mangle the shell config
     my $munged_rc;
     if ( my $rc = _find_shell_config() ) {
@@ -1853,8 +1912,7 @@ sub auto_mirrored_by {
     my $local = shift or return;
     local $|=1;
     $CPAN::Frontend->myprint("Looking for CPAN mirrors near you (please be patient)\n");
-    my $mirrors = CPAN::Mirrors->new;
-    $mirrors->parse_mirrored_by($local);
+    my $mirrors = CPAN::Mirrors->new($local);
 
     my $cnt = 0;
     my @best = $mirrors->best_mirrors(
